@@ -32,6 +32,7 @@ class Database(object):
     view_class = View
 
     def __init__(self, url_or_resource, *,
+                 dbname=None,
                  document_class=None,
                  design_document_class=None,
                  view_class=None):
@@ -45,9 +46,22 @@ class Database(object):
             url_or_resource = Resource(url_or_resource)
         self.resource = url_or_resource
         self._security = Security(self.resource)
+        self._dbname = dbname
+
+    def __getitem__(self, docid):
+        if docid.startswith('_design/'):
+            resource = self.resource(*docid.split('/', 1))
+            return self.design_document_class(resource, docid=docid)
+        else:
+            return self.document_class(self.resource(docid), docid=docid)
+
+    @property
+    def name(self):
+        """Returns a database name specified in class constructor."""
+        return self._dbname
 
     @asyncio.coroutine
-    def document(self, docid=None, *, auth=None, idfun=uuid.uuid4):
+    def doc(self, docid=None, *, auth=None, idfun=uuid.uuid4):
         """Returns :class:`~aiocouchdb.document.Document` instance against
         specified document ID.
 
@@ -68,18 +82,15 @@ class Database(object):
         """
         if docid is None:
             docid = str(idfun())
-        doc_resource = self.resource(docid)
-        resp = yield from doc_resource.head(auth=auth)
+        doc = self[docid]
+        resp = yield from doc.resource.head(auth=auth)
         if resp.status != 404:
             yield from resp.maybe_raise_error()
         yield from resp.read()
-        return self.document_class(doc_resource)
-
-    #: alias for :meth:`aiocouchdb.database.Database.document`
-    doc = document
+        return doc
 
     @asyncio.coroutine
-    def design_document(self, docid, *, auth=None):
+    def ddoc(self, docid, *, auth=None):
         """Returns :class:`~aiocouchdb.designdoc.DesignDocument` instance
         against specified document ID. This ID may startswith with ``_design/``
         prefix and if it's not prefix will be added automatically.
@@ -95,15 +106,12 @@ class Database(object):
         """
         if not docid.startswith('_design/'):
             docid = '_design/' + docid
-        doc_resource = self.resource(*docid.split('/', 1))
-        resp = yield from doc_resource.head(auth=auth)
+        ddoc = self[docid]
+        resp = yield from ddoc.resource.head(auth=auth)
         if resp.status != 404:
             yield from resp.maybe_raise_error()
         yield from resp.read()
-        return self.design_document_class(doc_resource)
-
-    #: alias for :meth:`aiocouchdb.database.Database.design_document`
-    ddoc = design_document
+        return ddoc
 
     @asyncio.coroutine
     def exists(self, *, auth=None):
@@ -167,18 +175,19 @@ class Database(object):
     @asyncio.coroutine
     def all_docs(self, *keys,
                  auth=None,
+                 feed_buffer_size=None,
                  att_encoding_info=None,
                  attachments=None,
                  conflicts=None,
                  descending=None,
-                 endkey=None,
+                 endkey=...,
                  endkey_docid=None,
                  include_docs=None,
                  inclusive_end=None,
                  limit=None,
                  skip=None,
                  stale=None,
-                 startkey=None,
+                 startkey=...,
                  startkey_docid=None,
                  update_seq=None):
         """Iterates over :ref:`all documents view <api/db/all_docs>`.
@@ -188,6 +197,7 @@ class Database(object):
                          amount of ``keys``
 
         :param auth: :class:`aiocouchdb.authn.AuthProvider` instance
+        :param int feed_buffer_size: Internal buffer size for fetched feed items
 
         :param bool att_encoding_info: Includes encoding information in an
                                        attachment stubs
@@ -217,10 +227,12 @@ class Database(object):
         :rtype: :class:`aiocouchdb.feeds.ViewFeed`
         """
         params = locals()
-        for key in ('self', 'auth'):
+        for key in ('self', 'auth', 'feed_buffer_size'):
             params.pop(key)
         view = self.view_class(self.resource('_all_docs'))
-        return (yield from view.request(auth=auth, params=params))
+        return (yield from view.request(auth=auth,
+                                        feed_buffer_size=feed_buffer_size,
+                                        params=params))
 
     def bulk_docs(self, docs, *, auth=None, all_or_nothing=None,
                   new_edits=None):
@@ -257,15 +269,18 @@ class Database(object):
 
     def changes(self, *doc_ids,
                 auth=None,
+                feed_buffer_size=None,
                 att_encoding_info=None,
                 attachments=None,
                 conflicts=None,
                 descending=None,
                 feed=None,
                 filter=None,
+                headers=None,
                 heartbeat=None,
                 include_docs=None,
                 limit=None,
+                params=None,
                 since=None,
                 style=None,
                 timeout=None,
@@ -279,6 +294,7 @@ class Database(object):
                             value.
 
         :param auth: :class:`aiocouchdb.authn.AuthProvider` instance
+        :param int feed_buffer_size: Internal buffer size for fetched feed items
 
         :param bool att_encoding_info: Includes encoding information in an
                                        attachment stubs
@@ -288,6 +304,7 @@ class Database(object):
         :param bool descending: Return changes in descending order
         :param str feed: :ref:`Changes feed type <changes>`
         :param str filter: Filter function name
+        :param dict headers: Custom request headers
         :param int heartbeat: Period in milliseconds after which an empty
                               line will be sent from server as the result
                               to keep connection alive
@@ -297,6 +314,7 @@ class Database(object):
                           value
         :param since: Starts listening changes feed since given
                       `update sequence` value
+        :param dict params: Custom request query parameters
         :param str style: Changes feed output style: ``all_docs``, ``main_only``
         :param int timeout: Period in milliseconds to await for new changes
                             before close the feed. Works for continuous feeds
@@ -305,22 +323,13 @@ class Database(object):
 
         :rtype: :class:`aiocouchdb.feeds.ChangesFeed`
         """
-        params = {}
-        maybe_set_param = (
-            lambda *kv: (None if kv[1] is None else params.update([kv])))
-        maybe_set_param('att_encoding_info', att_encoding_info)
-        maybe_set_param('attachments', attachments)
-        maybe_set_param('conflicts', conflicts)
-        maybe_set_param('descending', descending)
-        maybe_set_param('feed', feed)
-        maybe_set_param('filter', filter)
-        maybe_set_param('heartbeat', heartbeat)
-        maybe_set_param('include_docs', include_docs)
-        maybe_set_param('limit', limit)
-        maybe_set_param('since', since)
-        maybe_set_param('style', style)
-        maybe_set_param('timeout', timeout)
-        maybe_set_param('view', view)
+        print(locals())
+        params = dict(params or {})
+        params.update((key, value)
+                      for key, value in locals().items()
+                      if key not in {'self', 'doc_ids', 'auth', 'headers',
+                                     'params'}
+                      and value is not None)
 
         if doc_ids:
             data = {'doc_ids': doc_ids}
@@ -340,17 +349,17 @@ class Database(object):
                 assert params['filter'] == '_view'
 
         resp = yield from request('_changes', auth=auth, data=data,
-                                  params=params)
+                                  headers=headers, params=params)
         yield from resp.maybe_raise_error()
 
         if feed == 'continuous':
-            return ContinuousChangesFeed(resp)
+            return ContinuousChangesFeed(resp, buffer_size=feed_buffer_size)
         elif feed == 'eventsource':
-            return EventSourceChangesFeed(resp)
+            return EventSourceChangesFeed(resp, buffer_size=feed_buffer_size)
         elif feed == 'longpoll':
-            return LongPollChangesFeed(resp)
+            return LongPollChangesFeed(resp, buffer_size=feed_buffer_size)
         else:
-            return ChangesFeed(resp)
+            return ChangesFeed(resp, buffer_size=feed_buffer_size)
 
     @asyncio.coroutine
     def compact(self, ddoc_name=None, *, auth=None):
@@ -458,22 +467,23 @@ class Database(object):
     @asyncio.coroutine
     def temp_view(self, map_fun, red_fun=None, language=None, *,
                   auth=None,
+                  feed_buffer_size=None,
                   att_encoding_info=None,
                   attachments=None,
                   conflicts=None,
                   descending=None,
-                  endkey=None,
+                  endkey=...,
                   endkey_docid=None,
                   group=None,
                   group_level=None,
                   include_docs=None,
                   inclusive_end=None,
-                  keys=None,
+                  keys=...,
                   limit=None,
                   reduce=None,
                   skip=None,
                   stale=None,
-                  startkey=None,
+                  startkey=...,
                   startkey_docid=None,
                   update_seq=None):
         """Executes :ref:`temporary view <api/db/temp_view>` and returns
@@ -484,6 +494,7 @@ class Database(object):
         :param str language: Query server language to process the view
 
         :param auth: :class:`aiocouchdb.authn.AuthProvider` instance
+        :param int feed_buffer_size: Internal buffer size for fetched feed items
 
         :param bool att_encoding_info: Includes encoding information in an
                                        attachment stubs
@@ -491,7 +502,7 @@ class Database(object):
                                  **Warning**: use with caution!
         :param bool conflicts: Includes conflicts information into documents
         :param bool descending: Return rows in descending by key order
-        :param str endkey: Stop fetching rows when the specified key is reached
+        :param endkey: Stop fetching rows when the specified key is reached
         :param str endkey_docid: Stop fetching rows when the specified
                                  document ID is reached
         :param bool group: Reduces the view result grouping by unique keys
@@ -510,7 +521,7 @@ class Database(object):
         :param str stale: Allow to fetch the rows from a stale view, without
                           triggering index update. Supported values: ``ok``
                           and ``update_after``
-        :param str startkey: Return rows starting with the specified key
+        :param startkey: Return rows starting with the specified key
         :param str startkey_docid: Return rows starting with the specified
                                    document ID
         :param bool update_seq: Include an ``update_seq`` value into view
@@ -519,7 +530,8 @@ class Database(object):
         :rtype: :class:`aiocouchdb.feeds.ViewFeed`
         """
         params = locals()
-        for key in ('self', 'auth', 'map_fun', 'red_fun', 'language'):
+        for key in ('self', 'auth', 'map_fun', 'red_fun', 'language',
+                    'feed_buffer_size'):
             params.pop(key)
 
         data = {'map': map_fun}
@@ -529,7 +541,10 @@ class Database(object):
             data['language'] = language
 
         view = self.view_class(self.resource('_temp_view'))
-        return (yield from view.request(auth=auth, data=data, params=params))
+        return (yield from view.request(auth=auth,
+                                        feed_buffer_size=feed_buffer_size,
+                                        data=data,
+                                        params=params))
 
     @asyncio.coroutine
     def view_cleanup(self, *, auth=None):
@@ -568,11 +583,11 @@ class Security(object):
         if not secobj:
             secobj = {
                 'admins': {
-                    'users': [],
+                    'names': [],
                     'roles': []
                 },
                 'members': {
-                    'users': [],
+                    'names': [],
                     'roles': []
                 }
             }
@@ -609,12 +624,12 @@ class Security(object):
         yield from resp.maybe_raise_error()
         return (yield from resp.json())
 
-    def update_admins(self, *, auth=None, users=None, roles=None, merge=False):
+    def update_admins(self, *, auth=None, names=None, roles=None, merge=False):
         """Helper for :meth:`~aiocouchdb.database.Security.update` method to
         update only database administrators leaving members as is.
 
         :param auth: :class:`aiocouchdb.authn.AuthProvider` instance
-        :param list users: List of user names
+        :param list names: List of user names
         :param list roles: List of role names
         :param bool merge: Merges user/role lists with existed ones when
                            is ``True``, otherwise replaces them with the given
@@ -622,17 +637,17 @@ class Security(object):
         :rtype: dict
         """
         admins = {
-            'users': [] if users is None else users,
+            'names': [] if names is None else names,
             'roles': [] if roles is None else roles
         }
         return self.update(auth=auth, admins=admins, merge=merge)
 
-    def update_members(self, *, auth=None, users=None, roles=None, merge=False):
+    def update_members(self, *, auth=None, names=None, roles=None, merge=False):
         """Helper for :meth:`~aiocouchdb.database.Security.update` method to
         update only database members leaving administrators as is.
 
         :param auth: :class:`aiocouchdb.authn.AuthProvider` instance
-        :param list users: List of user names
+        :param list names: List of user names
         :param list roles: List of role names
         :param bool merge: Merges user/role lists with existed ones when
                            is ``True``, otherwise replaces them with the given
@@ -640,7 +655,7 @@ class Security(object):
         :rtype: dict
         """
         members = {
-            'users': [] if users is None else users,
+            'names': [] if names is None else names,
             'roles': [] if roles is None else roles
         }
         return self.update(auth=auth, members=members, merge=merge)
