@@ -23,20 +23,25 @@ class Document(object):
 
     attachment_class = Attachment
 
-    def __init__(self, url_or_resource, *, attachment_class=None):
+    def __init__(self, url_or_resource, *, docid=None, attachment_class=None):
         if attachment_class is not None:
             self.attachment_class = attachment_class
         if isinstance(url_or_resource, str):
             url_or_resource = Resource(url_or_resource)
         self.resource = url_or_resource
+        self._docid = docid
+
+    def __getitem__(self, attname):
+        resource = self.resource(*attname.split('/'))
+        return self.attachment_class(resource, name=attname)
 
     @property
     def id(self):
-        """Returns associated document ID."""
-        return self.resource.url.rsplit('/', 1)[-1]
+        """Returns a document id specified in class constructor."""
+        return self._docid
 
     @asyncio.coroutine
-    def attachment(self, attname, *, auth=None):
+    def att(self, attname, *, auth=None):
         """Returns :class:`~aiocouchdb.attachment.Attachment` instance against
         specified attachment.
 
@@ -49,15 +54,12 @@ class Document(object):
 
         :rtype: :attr:`aiocouchdb.document.Document.attachment_class`
         """
-        att_resource = self.resource(*attname.split('/'))
-        resp = yield from att_resource.head(auth=auth)
+        att = self[attname]
+        resp = yield from att.resource.head(auth=auth)
         if resp.status != 404:
             yield from resp.maybe_raise_error()
         yield from resp.read()
-        return self.attachment_class(att_resource)
-
-    #: alias for :meth:`aiocouchdb.document.Document.attachment`
-    att = attachment
+        return att
 
     @asyncio.coroutine
     def exists(self, rev=None, *, auth=None):
@@ -96,6 +98,20 @@ class Document(object):
         yield from resp.maybe_raise_error()
         return resp.status != 304
 
+    @asyncio.coroutine
+    def rev(self, *, auth=None):
+        """Returns current document revision by using `HEAD request`_.
+
+        :param auth: :class:`aiocouchdb.authn.AuthProvider` instance
+
+        :rtype: str
+
+        .. _HEAD request: http://docs.couchdb.org/en/latest/api/document/common.html#head--db-docid
+        """
+        resp = yield from self.resource.head(auth=auth)
+        yield from resp.maybe_raise_error()
+        return resp.headers['ETAG'].strip('"')
+
     @asyncio.coroutine
     def get(self, rev=None, *,
             auth=None,
@@ -135,20 +151,10 @@ class Document(object):
 
         .. _Returns a document: http://docs.couchdb.org/en/latest/api/document/common.html#get--db-docid
         """
-        params = {}
-        maybe_set_param = (
-            lambda *kv: (None if kv[1] is None else params.update([kv])))
-        maybe_set_param('att_encoding_info', att_encoding_info)
-        maybe_set_param('attachments', attachments)
-        maybe_set_param('atts_since', atts_since)
-        maybe_set_param('conflicts', conflicts)
-        maybe_set_param('deleted_conflicts', deleted_conflicts)
-        maybe_set_param('local_seq', local_seq)
-        maybe_set_param('meta', meta)
-        maybe_set_param('open_revs', open_revs)
-        maybe_set_param('rev', rev)
-        maybe_set_param('revs', revs)
-        maybe_set_param('revs_info', revs_info)
+        params = dict((key, value)
+                      for key, value in locals().items()
+                      if key not in {'self', 'auth'} and
+                         value is not None)
 
         if atts_since is not None:
             params['atts_since'] = json.dumps(atts_since)
@@ -192,13 +198,10 @@ class Document(object):
 
         :rtype: :class:`~aiocouchdb.document.OpenRevsMultipartReader`
         """
-        params = {}
-        maybe_set_param = (
-            lambda *kv: (None if kv[1] is None else params.update([kv])))
-        maybe_set_param('att_encoding_info', att_encoding_info)
-        maybe_set_param('atts_since', atts_since)
-        maybe_set_param('local_seq', local_seq)
-        maybe_set_param('revs', revs)
+        params = dict((key, value)
+                      for key, value in locals().items()
+                      if key not in {'self', 'auth'} and
+                         value is not None)
 
         if atts_since is not None:
             params['atts_since'] = json.dumps(atts_since)
@@ -250,18 +253,11 @@ class Document(object):
 
         :rtype: :class:`~aiocouchdb.document.DocAttachmentsMultipartReader`
         """
-        params = {'attachments': True}
-        maybe_set_param = (
-            lambda *kv: (None if kv[1] is None else params.update([kv])))
-        maybe_set_param('att_encoding_info', att_encoding_info)
-        maybe_set_param('atts_since', atts_since)
-        maybe_set_param('conflicts', conflicts)
-        maybe_set_param('deleted_conflicts', deleted_conflicts)
-        maybe_set_param('local_seq', local_seq)
-        maybe_set_param('meta', meta)
-        maybe_set_param('rev', rev)
-        maybe_set_param('revs', revs)
-        maybe_set_param('revs_info', revs_info)
+        params = dict((key, value)
+                      for key, value in locals().items()
+                      if key not in {'self', 'auth'} and
+                         value is not None)
+        params['attachments'] = True
 
         if atts_since is not None:
             params['atts_since'] = json.dumps(atts_since)
@@ -314,12 +310,10 @@ class Document(object):
 
         .. _Updates a document: http://docs.couchdb.org/en/latest/api/document/common.html#put--db-docid
         """
-        params = {}
-        maybe_set_param = (
-            lambda *kv: (None if kv[1] is None else params.update([kv])))
-        maybe_set_param('batch', batch)
-        maybe_set_param('new_edits', new_edits)
-        maybe_set_param('rev', rev)
+        params = dict((key, value)
+                      for key, value in locals().items()
+                      if key not in {'self', 'doc', 'auth'} and
+                         value is not None)
 
         if not isinstance(doc, Mapping):
             raise TypeError('Mapping instance expected, dict - preferred')
@@ -334,7 +328,7 @@ class Document(object):
         return (yield from resp.json())
 
     @asyncio.coroutine
-    def remove(self, rev, *, auth=None, preserve_content=None):
+    def delete(self, rev, *, auth=None, preserve_content=None):
         """`Deletes a document`_ from server.
 
         By default document will be deleted using `DELETE` HTTP method.
